package com.anabas.eventbus;

/**
 * Title:        Collaboration Framework
 * Description:  This project contains the Anabas, Inc. collaboration framework libraries.
 * Copyright:    Copyright (c) 2000
 * Company:      Anabas, Inc.
 * @author John Yin
 * @version 1.0
 */

public class CodingSamples {

  /**
   * Done by our application framework before any collaboration application's
   * initialization.
   */
  public void FrameworkSetup() {
    User user = null; // Somehow get the user object associated with this instance.

    StreamContextFactory streamCtxFactory = new StreamContextFactory();

    /**
     * Setup pre-connection initialization parameters including:
     * - Provider specific setup such as server address.
     * - Anabas specific setup parameters such as:
     *   (1) Authentication parameters [ Future ]
     *   (2) Root context of the connection in the Anabas global bus directory.
     *
     */
    streamCtxFactory.setProperty("server","...");
    streamCtxFactory.setProperty("authentication","...");

    // Set the user object for this context.  Future filters for subscription
    // must contain the user field.  All createSubscriber() instances returned
    // will have an implicit "user=XXX" field.
    streamCtxFactory.setProperty("user",user);

    // Get the root context of this event bus instance.
    StreamContext ctx = streamCtxFactory.findContext("/global/newsgroups/stream/cs101");

    // Setup the Anabas collaborative framework environment so applications can
    // locate this service.
    initialContext.bind("services/EventBus",ctx);
  }

  // A sample whiteboard application initialization
  public void WhitboardInit() {
    InitialContext ctx = new InitialContext();

    // Lookup the event bus service which is guaranteed to exist by the framework
    StreamContext sctx = (StreamContext)ctx.lookup("services/EventBus");

    // Find the stream associated with a sharedlet type.
    Stream stream = sctx.findStream("application/x-sharedlet-whiteboard");
    if (stream == null) {
      stream = sctx.createStream("application/x-sharedlet-whiteboard");
    }

    Publisher  myPublisher      = stream.createSubscriber();
    Subscriber mySubscriber = Stream.createPublisher();
  }

  public void WhiteboardDraw() {
    // Assume the existence of myStream, a reference to the created event bus
    // Stream application whiteboard.
    int messageType = Stream.MESSAGETYPE_XXX;
    Message message = myPublisher.createMessage(Message.MSGTYPE_MINOR,Message.BODYTYPE_XML);

    // Set some headers, set some properties & set the body.
    myPublisher.broadcast(message);
  }

  public void IMInit() {
    InitialContext ctx = new InitialContext();

    user = (User)ctx.lookup("session/user");

    // Find the stream associated with a sharedlet type.
    Stream stream = sctx.findStream("application/x-sharedlet-im");
    if (stream == null) {
      stream = sctx.createStream("application/x-sharedlet-im");
    }

    Publisher  myPublisher  = Stream.createPublisher();
    Subscriber mySubscriber = Stream.createSubscriber(d);
  }

  public void IMSend(String userID) {
    // UserSharedletDestinationSelector is a convenience class for selecting user property
    // matches.
     // Some XML query language based selector.
    DestinationSelector ds = new UserSharedletDestSelector(user);
    Destination d = stream.findDestination(ds);

    // Assume the existence of myStream, a reference to the created event bus
    // Stream application whiteboard.
    int messageType = Stream.MESSAGETYPE_XXX;
    Message message = myPublisher.createMessage(Message.MSGTYPE_MINOR,Message.BODYTYPE_Text);

    // Set some headers, set some properties & set the body.

    // Now send it to the right person.
    myPublisher.unicast(m,d);
  }

  // Typically the sharedlet session logic initialization steps.
  public void SharedDisplay_ServerInit() {
    InitialContext ctx = new InitialContext();

    // Lookup the event bus service which is guaranteed to exist by the framework
    StreamContext sctx = (StreamContext)ctx.lookup("services/EventBus");
    // Since only expect 1 instance of this particular session logic to be running
    // no need to check for prior existence.  This Stream is used for clients
    // who want to send the session logic information.
    StreamContext mainContext = sctx.createStreamContext("application/x-sharedlet-im");
    // This Stream is the standard session logic => broadcast Stream.  A
    // shared display session logic component may create 3 Streams, 1 for each
    // resolution: 1280x1024,1024x768,Windows CE.  The appropriate device would
    // subscriber to the appropriate resolution Stream.
    stream[HIGH] = mainContext.createStream("high_resolution");
    stream[MEDIUM] = mainContext.createStream("medium_resolution");
    stream[PDA] = mainContext.createStream("pda_resolution");
    stream[SERVER_PORT] = mainContext.createStream("server_port");

    Publisher publisher[HIGH] = stream[HIGH].createPublisher();
    Publisher publisher[MEDIUM] = stream[MEDIUM].createPublisher();
    Publisher publisher[PDA] = stream[PDA].createPublisher();

    Subscriber mySubscriber = stream[SERVER_PORT].createSubscriber();
  }

  /**
   * Receive frames of shared display, optimize it for each resolution level
   * supported and send it on to the appropriate clients.
   */

  public SharedDisplay_ServerLogic() {
    // mySubscriber receives some data.
    Message message = mySubscriber.receiveMessage();
    Destination   d = message.getProperty("From");
    DestinationSelector s = new XMLQuerySelector("Some string w/ everyone except the from destination");

    // !!DECIDE!!
    // Should really cache these destinations.  Alternatively, we can create
    // a concept of global destination that works for all Streams within the
    // context of a event bus instance.
    Destination target[] = new Destionation[3];
    for(int res=HIGH;res < PDA;res++) {
      target[res] = stream[res].findDestination(s);
    }

    for(int res=HIGH;res < PDA;res++) {
      if (stream[HIGH] == null) continue;
      Message newMsg = optimize(message,res);
      Publisher[res].multicast(newMsg,target[res]);
    }
  }

  public SharedDisplay_ClientInit() {
    InitialContext ctx = new InitialContext();

    // Lookup the event bus service which is guaranteed to exist by the framework
    StreamContext sctx = (StreamContext)ctx.lookup("services/EventBus");
    // No need to check for Stream existence.  Assume server was instantiated.
    StreamContext streamContext = sctx.findStreamContext("application/x-sharedlet-im");
    Stream streamPDA = streamContext.findStream("pda_resolution");

    Publisher myPublisher = streamPDA.createPublisher();
    Subscriber mySubscriber = streamPDA.createSubscriber();
  }

}










